@@ -26,26 +26,24 @@
 
 #include "config.h"
 
-#include "gdkvisual.h"
-
-#include "gdkx.h"
+#include "gdkvisualprivate.h"
 #include "gdkprivate-x11.h"
 #include "gdkscreen-x11.h"
-#include "gdkinternals.h"
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 
-struct _GdkVisualPrivate
+struct _GdkX11Visual
 {
+  GdkVisual visual;
+
   Visual *xvisual;
-  GdkScreen *screen;
   Colormap colormap;
 };
 
-struct _GdkVisualClass
+struct _GdkX11VisualClass
 {
-  GObjectClass parent_class;
+  GdkVisualClass visual_class;
 };
 
 static void     gdk_visual_add            (GdkVisual *visual);
@@ -71,42 +69,36 @@ static const gchar *const visual_names[] =
 
 #endif /* G_ENABLE_DEBUG */
 
-G_DEFINE_TYPE (GdkVisual, gdk_visual, G_TYPE_OBJECT)
+G_DEFINE_TYPE (GdkX11Visual, gdk_x11_visual, GDK_TYPE_VISUAL)
 
 static void
-gdk_visual_finalize (GObject *object)
+gdk_x11_visual_init (GdkX11Visual *x11_visual)
 {
-  GdkVisualPrivate *priv = (GdkVisualPrivate *) object;
-
-  if (priv->colormap != None)
-    XFreeColormap (GDK_SCREEN_XDISPLAY (priv->screen),
-                   priv->colormap);
-
-  G_OBJECT_CLASS (gdk_visual_parent_class)->finalize (object);
+  x11_visual->colormap = None;
 }
 
 static void
-gdk_visual_class_init (GdkVisualClass *visual_class)
+gdk_x11_visual_finalize (GObject *object)
 {
-  GObjectClass *object_class = G_OBJECT_CLASS (visual_class);
+  GdkVisual *visual = (GdkVisual *)object;
+  GdkX11Visual *x11_visual = (GdkX11Visual *)object;
 
-  g_type_class_add_private (object_class, sizeof (GdkVisualPrivate));
+  if (x11_visual->colormap != None)
+    XFreeColormap (GDK_SCREEN_XDISPLAY (visual->screen), x11_visual->colormap);
 
-  object_class->finalize = gdk_visual_finalize;
+  G_OBJECT_CLASS (gdk_x11_visual_parent_class)->finalize (object);
 }
 
 static void
-gdk_visual_init (GdkVisual *visual)
+gdk_x11_visual_class_init (GdkX11VisualClass *class)
 {
-  visual->priv = G_TYPE_INSTANCE_GET_PRIVATE (visual,
-                                              GDK_TYPE_VISUAL,
-                                              GdkVisualPrivate);
+  GObjectClass *object_class = G_OBJECT_CLASS (class);
 
-  visual->priv->colormap = None;
+  object_class->finalize = gdk_x11_visual_finalize;
 }
 
 void
-_gdk_visual_init (GdkScreen *screen)
+_gdk_x11_screen_init_visuals (GdkScreen *screen)
 {
   static const gint possible_depths[8] = { 32, 30, 24, 16, 15, 8, 4, 1 };
   static const GdkVisualType possible_types[6] =
@@ -119,7 +111,7 @@ _gdk_visual_init (GdkScreen *screen)
       GDK_VISUAL_STATIC_GRAY
     };
 
-  GdkScreenX11 *screen_x11;
+  GdkX11Screen *x11_screen;
   XVisualInfo *visual_list;
   XVisualInfo visual_template;
   GdkVisual *temp_visual;
@@ -128,25 +120,25 @@ _gdk_visual_init (GdkScreen *screen)
   int nxvisuals;
   int nvisuals;
   int i, j;
-  
+
   g_return_if_fail (GDK_IS_SCREEN (screen));
-  screen_x11 = GDK_SCREEN_X11 (screen);
+  x11_screen = GDK_X11_SCREEN (screen);
 
   nxvisuals = 0;
-  visual_template.screen = screen_x11->screen_num;
-  visual_list = XGetVisualInfo (screen_x11->xdisplay, VisualScreenMask, &visual_template, &nxvisuals);
-  
+  visual_template.screen = x11_screen->screen_num;
+  visual_list = XGetVisualInfo (x11_screen->xdisplay, VisualScreenMask, &visual_template, &nxvisuals);
+
   visuals = g_new (GdkVisual *, nxvisuals);
   for (i = 0; i < nxvisuals; i++)
-    visuals[i] = g_object_new (GDK_TYPE_VISUAL, NULL);
+    visuals[i] = g_object_new (GDK_TYPE_X11_VISUAL, NULL);
 
-  default_xvisual = DefaultVisual (screen_x11->xdisplay, screen_x11->screen_num);
+  default_xvisual = DefaultVisual (x11_screen->xdisplay, x11_screen->screen_num);
 
   nvisuals = 0;
   for (i = 0; i < nxvisuals; i++)
     {
-      visuals[nvisuals]->priv->screen = screen;
-      
+      visuals[nvisuals]->screen = screen;
+
       if (visual_list[i].depth >= 1)
 	{
 #ifdef __cplusplus
@@ -177,14 +169,14 @@ _gdk_visual_init (GdkScreen *screen)
 
 	  visuals[nvisuals]->depth = visual_list[i].depth;
 	  visuals[nvisuals]->byte_order =
-	    (ImageByteOrder(screen_x11->xdisplay) == LSBFirst) ?
+	    (ImageByteOrder(x11_screen->xdisplay) == LSBFirst) ?
 	    GDK_LSB_FIRST : GDK_MSB_FIRST;
 	  visuals[nvisuals]->red_mask = visual_list[i].red_mask;
 	  visuals[nvisuals]->green_mask = visual_list[i].green_mask;
 	  visuals[nvisuals]->blue_mask = visual_list[i].blue_mask;
 	  visuals[nvisuals]->colormap_size = visual_list[i].colormap_size;
 	  visuals[nvisuals]->bits_per_rgb = visual_list[i].bits_per_rgb;
-	  visuals[nvisuals]->priv->xvisual = visual_list[i].visual;
+	  GDK_X11_VISUAL (visuals[nvisuals])->xvisual = visual_list[i].visual;
 
 	  if ((visuals[nvisuals]->type == GDK_VISUAL_TRUE_COLOR) ||
 	      (visuals[nvisuals]->type == GDK_VISUAL_DIRECT_COLOR))
@@ -215,7 +207,7 @@ _gdk_visual_init (GdkScreen *screen)
 	      visuals[nvisuals]->blue_shift = 0;
 	      visuals[nvisuals]->blue_prec = 0;
 	    }
-	  
+
 	  nvisuals += 1;
 	}
     }
@@ -259,11 +251,11 @@ _gdk_visual_init (GdkScreen *screen)
 
   for (i = 0; i < nvisuals; i++)
     {
-      if (default_xvisual->visualid == visuals[i]->priv->xvisual->visualid)
+      if (default_xvisual->visualid == GDK_X11_VISUAL (visuals[i])->xvisual->visualid)
          {
- 	   screen_x11->system_visual = visuals[i];
-           visuals[i]->priv->colormap = DefaultColormap (screen_x11->xdisplay,
-                                                         screen_x11->screen_num);
+           x11_screen->system_visual = visuals[i];
+           GDK_X11_VISUAL (visuals[i])->colormap =
+               DefaultColormap (x11_screen->xdisplay, x11_screen->screen_num);
          }
 
       /* For now, we only support 8888 ARGB for the "rgba visual".
@@ -275,11 +267,11 @@ _gdk_visual_init (GdkScreen *screen)
 	   visuals[i]->green_mask == 0x00ff00 &&
 	   visuals[i]->blue_mask  == 0x0000ff))
 	{
-	  screen_x11->rgba_visual = GDK_VISUAL (visuals[i]);
-	}
+	  x11_screen->rgba_visual = visuals[i];
+        }
     }
 
-#ifdef G_ENABLE_DEBUG 
+#ifdef G_ENABLE_DEBUG
   if (_gdk_debug_flags & GDK_DEBUG_MISC)
     {
       static const gchar *const visual_names[] =
@@ -293,282 +285,172 @@ _gdk_visual_init (GdkScreen *screen)
       };
 
       for (i = 0; i < nvisuals; i++)
-        g_message ("visual: %s: %d",
-                   visual_names[visuals[i]->type],
-                   visuals[i]->depth);
+        g_message ("visual: %s: %d", visual_names[visuals[i]->type], visuals[i]->depth);
     }
 #endif /* G_ENABLE_DEBUG */
 
-  screen_x11->navailable_depths = 0;
+  x11_screen->navailable_depths = 0;
   for (i = 0; i < G_N_ELEMENTS (possible_depths); i++)
     {
       for (j = 0; j < nvisuals; j++)
 	{
 	  if (visuals[j]->depth == possible_depths[i])
 	    {
-	      screen_x11->available_depths[screen_x11->navailable_depths++] = visuals[j]->depth;
+	      x11_screen->available_depths[x11_screen->navailable_depths++] = visuals[j]->depth;
 	      break;
 	    }
 	}
     }
 
-  if (screen_x11->navailable_depths == 0)
+  if (x11_screen->navailable_depths == 0)
     g_error ("unable to find a usable depth");
 
-  screen_x11->navailable_types = 0;
+  x11_screen->navailable_types = 0;
   for (i = 0; i < G_N_ELEMENTS (possible_types); i++)
     {
       for (j = 0; j < nvisuals; j++)
 	{
 	  if (visuals[j]->type == possible_types[i])
 	    {
-	      screen_x11->available_types[screen_x11->navailable_types++] = visuals[j]->type;
+	      x11_screen->available_types[x11_screen->navailable_types++] = visuals[j]->type;
 	      break;
 	    }
 	}
     }
 
   for (i = 0; i < nvisuals; i++)
-    gdk_visual_add ((GdkVisual*) visuals[i]);
+    gdk_visual_add (visuals[i]);
 
-  if (screen_x11->navailable_types == 0)
+  if (x11_screen->navailable_types == 0)
     g_error ("unable to find a usable visual type");
 
-  screen_x11->visuals = visuals;
-  screen_x11->nvisuals = nvisuals;
+  x11_screen->visuals = visuals;
+  x11_screen->nvisuals = nvisuals;
 }
 
-/**
- * gdk_visual_get_best_depth:
- * 
- * Get the best available depth for the default GDK screen.  "Best"
- * means "largest," i.e. 32 preferred over 24 preferred over 8 bits
- * per pixel.
- * 
- * Return value: best available depth
- **/
 gint
-gdk_visual_get_best_depth (void)
+_gdk_x11_screen_visual_get_best_depth (GdkScreen *screen)
 {
-  GdkScreen *screen = gdk_screen_get_default();
-  
-  return GDK_SCREEN_X11 (screen)->available_depths[0];
+  return GDK_X11_SCREEN (screen)->available_depths[0];
 }
 
-/**
- * gdk_visual_get_best_type:
- * 
- * Return the best available visual type for the default GDK screen.
- * 
- * Return value: best visual type
- **/
 GdkVisualType
-gdk_visual_get_best_type (void)
+_gdk_x11_screen_visual_get_best_type (GdkScreen *screen)
 {
-  GdkScreen *screen = gdk_screen_get_default();
-  
-  return GDK_SCREEN_X11 (screen)->available_types[0];
+  return GDK_X11_SCREEN (screen)->available_types[0];
 }
 
-/**
- * gdk_screen_get_system_visual:
- * @screen: a #GdkScreen.
- * 
- * Get the system's default visual for @screen.
- * This is the visual for the root window of the display.
- * The return value should not be freed.
- * 
- * Return value: (transfer none): the system visual
- *
- * Since: 2.2
- **/
 GdkVisual *
-gdk_screen_get_system_visual (GdkScreen * screen)
+_gdk_x11_screen_get_system_visual (GdkScreen *screen)
 {
   g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
 
-  return ((GdkVisual *) GDK_SCREEN_X11 (screen)->system_visual);
+  return ((GdkVisual *) GDK_X11_SCREEN (screen)->system_visual);
 }
 
-/**
- * gdk_visual_get_best:
- *
- * Get the visual with the most available colors for the default
- * GDK screen. The return value should not be freed.
- * 
- * Return value: (transfer none): best visual
- **/
 GdkVisual*
-gdk_visual_get_best (void)
+_gdk_x11_screen_visual_get_best (GdkScreen *screen)
 {
-  GdkScreenX11 *screen_x11 = GDK_SCREEN_X11 (gdk_screen_get_default());
+  GdkX11Screen *x11_screen = GDK_X11_SCREEN (screen);
 
-  return (GdkVisual *)screen_x11->visuals[0];
+  return x11_screen->visuals[0];
 }
 
-/**
- * gdk_visual_get_best_with_depth:
- * @depth: a bit depth
- * 
- * Get the best visual with depth @depth for the default GDK screen.
- * Color visuals and visuals with mutable colormaps are preferred
- * over grayscale or fixed-colormap visuals. The return value should not
- * be freed. %NULL may be returned if no visual supports @depth.
- * 
- * Return value: (transfer none): best visual for the given depth
- **/
 GdkVisual*
-gdk_visual_get_best_with_depth (gint depth)
+_gdk_x11_screen_visual_get_best_with_depth (GdkScreen *screen,
+                                            gint       depth)
 {
-  GdkScreenX11 *screen_x11 = GDK_SCREEN_X11 (gdk_screen_get_default ());
+  GdkX11Screen *x11_screen = GDK_X11_SCREEN (screen);
   GdkVisual *return_val;
   int i;
   
   return_val = NULL;
-  for (i = 0; i < screen_x11->nvisuals; i++)
-    if (depth == screen_x11->visuals[i]->depth)
+  for (i = 0; i < x11_screen->nvisuals; i++)
+    if (depth == x11_screen->visuals[i]->depth)
       {
-	return_val = (GdkVisual *) screen_x11->visuals[i];
+	return_val = x11_screen->visuals[i];
 	break;
       }
 
   return return_val;
 }
 
-/**
- * gdk_visual_get_best_with_type:
- * @visual_type: a visual type
- *
- * Get the best visual of the given @visual_type for the default GDK screen.
- * Visuals with higher color depths are considered better. The return value
- * should not be freed. %NULL may be returned if no visual has type
- * @visual_type.
- * 
- * Return value: (transfer none): best visual of the given type
- **/
 GdkVisual*
-gdk_visual_get_best_with_type (GdkVisualType visual_type)
+_gdk_x11_screen_visual_get_best_with_type (GdkScreen     *screen,
+                                           GdkVisualType  visual_type)
 {
-  GdkScreenX11 *screen_x11 = GDK_SCREEN_X11 (gdk_screen_get_default ());
+  GdkX11Screen *x11_screen = GDK_X11_SCREEN (screen);
   GdkVisual *return_val;
   int i;
 
   return_val = NULL;
-  for (i = 0; i < screen_x11->nvisuals; i++)
-    if (visual_type == screen_x11->visuals[i]->type)
+  for (i = 0; i < x11_screen->nvisuals; i++)
+    if (visual_type == x11_screen->visuals[i]->type)
       {
-	return_val = (GdkVisual *) screen_x11->visuals[i];
+	return_val = x11_screen->visuals[i];
 	break;
       }
 
   return return_val;
 }
 
-/**
- * gdk_visual_get_best_with_both:
- * @depth: a bit depth
- * @visual_type: a visual type
- *
- * Combines gdk_visual_get_best_with_depth() and gdk_visual_get_best_with_type().
- * 
- * Return value: (transfer none): best visual with both @depth and
- *     @visual_type, or %NULL if none
- **/
 GdkVisual*
-gdk_visual_get_best_with_both (gint          depth,
-			       GdkVisualType visual_type)
+_gdk_x11_screen_visual_get_best_with_both (GdkScreen     *screen,
+                                           gint           depth,
+                                           GdkVisualType  visual_type)
 {
-  GdkScreenX11 *screen_x11 = GDK_SCREEN_X11 (gdk_screen_get_default ());
+  GdkX11Screen *x11_screen = GDK_X11_SCREEN (screen);
   GdkVisual *return_val;
   int i;
 
   return_val = NULL;
-  for (i = 0; i < screen_x11->nvisuals; i++)
-    if ((depth == screen_x11->visuals[i]->depth) &&
-	(visual_type == screen_x11->visuals[i]->type))
+  for (i = 0; i < x11_screen->nvisuals; i++)
+    if ((depth == x11_screen->visuals[i]->depth) &&
+	(visual_type == x11_screen->visuals[i]->type))
       {
-	return_val = (GdkVisual *) screen_x11->visuals[i];
+	return_val = x11_screen->visuals[i];
 	break;
       }
 
   return return_val;
 }
 
-/**
- * gdk_query_depths:
- * @depths: (out) (array): return location for available depths
- * @count: (out): return location for number of available depths
- *
- * This function returns the available bit depths for the default
- * screen. It's equivalent to listing the visuals
- * (gdk_list_visuals()) and then looking at the depth field in each
- * visual, removing duplicates.
- * 
- * The array returned by this function should not be freed.
- * 
- **/
 void
-gdk_query_depths  (gint **depths,
-		   gint  *count)
+_gdk_x11_screen_query_depths  (GdkScreen  *screen,
+                               gint      **depths,
+                               gint       *count)
 {
-  GdkScreenX11 *screen_x11 = GDK_SCREEN_X11 (gdk_screen_get_default ());
-  
-  *count = screen_x11->navailable_depths;
-  *depths = screen_x11->available_depths;
+  GdkX11Screen *x11_screen = GDK_X11_SCREEN (screen);
+
+  *count = x11_screen->navailable_depths;
+  *depths = x11_screen->available_depths;
 }
 
-/**
- * gdk_query_visual_types:
- * @visual_types: return location for the available visual types
- * @count: return location for the number of available visual types
- *
- * This function returns the available visual types for the default
- * screen. It's equivalent to listing the visuals
- * (gdk_list_visuals()) and then looking at the type field in each
- * visual, removing duplicates.
- * 
- * The array returned by this function should not be freed.
- **/
 void
-gdk_query_visual_types (GdkVisualType **visual_types,
-			gint           *count)
+_gdk_x11_screen_query_visual_types (GdkScreen      *screen,
+                                    GdkVisualType **visual_types,
+                                    gint           *count)
 {
-  GdkScreenX11 *screen_x11 = GDK_SCREEN_X11 (gdk_screen_get_default ());
-  
-  *count = screen_x11->navailable_types;
-  *visual_types = screen_x11->available_types;
+  GdkX11Screen *x11_screen = GDK_X11_SCREEN (screen);
+
+  *count = x11_screen->navailable_types;
+  *visual_types = x11_screen->available_types;
 }
 
-/**
- * gdk_screen_list_visuals:
- * @screen: the relevant #GdkScreen.
- *  
- * Lists the available visuals for the specified @screen.
- * A visual describes a hardware image data format.
- * For example, a visual might support 24-bit color, or 8-bit color,
- * and might expect pixels to be in a certain format.
- *
- * Call g_list_free() on the return value when you're finished with it.
- * 
- * Return value: (transfer container) (element-type GdkVisual):
- *     a list of visuals; the list must be freed, but not its contents
- *
- * Since: 2.2
- **/
 GList *
-gdk_screen_list_visuals (GdkScreen *screen)
+_gdk_x11_screen_list_visuals (GdkScreen *screen)
 {
   GList *list;
-  GdkScreenX11 *screen_x11;
+  GdkX11Screen *x11_screen;
   guint i;
 
   g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
-  screen_x11 = GDK_SCREEN_X11 (screen);
-  
+  x11_screen = GDK_X11_SCREEN (screen);
+
   list = NULL;
 
-  for (i = 0; i < screen_x11->nvisuals; ++i)
-    list = g_list_append (list, screen_x11->visuals[i]);
+  for (i = 0; i < x11_screen->nvisuals; ++i)
+    list = g_list_append (list, x11_screen->visuals[i]);
 
   return list;
 }
@@ -587,50 +469,36 @@ gdk_screen_list_visuals (GdkScreen *screen)
  */
 GdkVisual *
 gdk_x11_screen_lookup_visual (GdkScreen *screen,
-			      VisualID   xvisualid)
+                              VisualID   xvisualid)
 {
   int i;
-  GdkScreenX11 *screen_x11;
+  GdkX11Screen *x11_screen;
   g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
-  screen_x11 = GDK_SCREEN_X11 (screen);
+  x11_screen = GDK_X11_SCREEN (screen);
 
-  for (i = 0; i < screen_x11->nvisuals; i++)
-    if (xvisualid == screen_x11->visuals[i]->priv->xvisual->visualid)
-      return (GdkVisual *)  screen_x11->visuals[i];
+  for (i = 0; i < x11_screen->nvisuals; i++)
+    if (xvisualid == GDK_X11_VISUAL (x11_screen->visuals[i])->xvisual->visualid)
+      return x11_screen->visuals[i];
 
   return NULL;
 }
 
-/**
- * gdkx_visual_get:
- * @xvisualid: a X visual id.
- * 
- * Returns a #GdkVisual corresponding to a X visual. 
- * 
- * Return value: the #GdkVisual.
- **/
-GdkVisual*
-gdkx_visual_get (VisualID xvisualid)
-{
-  return gdk_x11_screen_lookup_visual (gdk_screen_get_default (), xvisualid);
-}
-
 static void
 gdk_visual_add (GdkVisual *visual)
 {
-  GdkScreenX11 *screen_x11 = GDK_SCREEN_X11 (visual->priv->screen);
-  
-  if (!screen_x11->visual_hash)
-    screen_x11->visual_hash = g_hash_table_new ((GHashFunc) gdk_visual_hash,
+  GdkX11Screen *x11_screen = GDK_X11_SCREEN (visual->screen);
+
+  if (!x11_screen->visual_hash)
+    x11_screen->visual_hash = g_hash_table_new ((GHashFunc) gdk_visual_hash,
                                                 (GEqualFunc) gdk_visual_equal);
 
-  g_hash_table_insert (screen_x11->visual_hash, visual->priv->xvisual, visual);
+  g_hash_table_insert (x11_screen->visual_hash, GDK_X11_VISUAL (visual)->xvisual, visual);
 }
 
 static void
 gdk_visual_decompose_mask (gulong  mask,
-			   gint   *shift,
-			   gint   *prec)
+                           gint   *shift,
+                           gint   *prec)
 {
   *shift = 0;
   *prec = 0;
@@ -662,7 +530,7 @@ gdk_visual_hash (Visual *key)
 
 static gboolean
 gdk_visual_equal (Visual *a,
-		  Visual *b)
+                  Visual *b)
 {
   return (a->visualid == b->visualid);
 }
@@ -678,53 +546,35 @@ gdk_visual_equal (Visual *a,
 Colormap
 _gdk_visual_get_x11_colormap (GdkVisual *visual)
 {
-  GdkVisualPrivate *priv;
+  GdkX11Visual *x11_visual;
 
   g_return_val_if_fail (GDK_IS_VISUAL (visual), None);
 
-  priv = visual->priv;
+  x11_visual = GDK_X11_VISUAL (visual);
 
-  if (priv->colormap == None)
+  if (x11_visual->colormap == None)
     {
-      priv->colormap = XCreateColormap (GDK_SCREEN_XDISPLAY (priv->screen),
-                                        GDK_SCREEN_XROOTWIN (priv->screen),
-                                        GDK_VISUAL_XVISUAL (visual),
-                                        AllocNone);
+      x11_visual->colormap = XCreateColormap (GDK_SCREEN_XDISPLAY (visual->screen),
+                                              GDK_SCREEN_XROOTWIN (visual->screen),
+                                              x11_visual->xvisual,
+                                              AllocNone);
     }
 
-  return priv->colormap;
+  return x11_visual->colormap;
 }
 
 /**
  * gdk_x11_visual_get_xvisual:
  * @visual: a #GdkVisual.
- * 
+ *
  * Returns the X visual belonging to a #GdkVisual.
- * 
+ *
  * Return value: an Xlib <type>Visual*</type>.
  **/
 Visual *
 gdk_x11_visual_get_xvisual (GdkVisual *visual)
-{
-  g_return_val_if_fail (visual != NULL, NULL);
-
-  return visual->priv->xvisual;
-}
-
-/**
- * gdk_visual_get_screen:
- * @visual: a #GdkVisual
- * 
- * Gets the screen to which this visual belongs
- * 
- * Return value: (transfer none): the screen to which this visual belongs.
- *
- * Since: 2.2
- **/
-GdkScreen *
-gdk_visual_get_screen (GdkVisual *visual)
 {
   g_return_val_if_fail (GDK_IS_VISUAL (visual), NULL);
 
-  return visual->priv->screen;
+  return GDK_X11_VISUAL (visual)->xvisual;
 }
