@@ -45,16 +45,16 @@ G_BEGIN_DECLS
 #define GTK_SCALE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_SCALE, GtkScaleClass))
 
 
-typedef struct _GtkScale        GtkScale;
-typedef struct _GtkScalePriv    GtkScalePriv;
-typedef struct _GtkScaleClass   GtkScaleClass;
+typedef struct _GtkScale              GtkScale;
+typedef struct _GtkScalePrivate       GtkScalePrivate;
+typedef struct _GtkScaleClass         GtkScaleClass;
 
 struct _GtkScale
 {
   GtkRange range;
 
-  /* <private> */
-  GtkScalePriv *priv;
+  /*< private >*/
+  GtkScalePrivate *priv;
 };
 
 struct _GtkScaleClass
@@ -74,6 +74,7 @@ struct _GtkScaleClass
   void (*_gtk_reserved1) (void);
   void (*_gtk_reserved2) (void);
   void (*_gtk_reserved3) (void);
+  void (*_gtk_reserved4) (void);
 };
 
 GType             gtk_scale_get_type           (void) G_GNUC_CONST;
