@@ -1,6 +1,7 @@
 /* Scrolling test suite for GtkTreeView
  * Copyright (C) 2006  Kristian Rietveld  <kris@gtk.org>
  * Copyright (C) 2007  Imendio AB,  Kristian Rietveld
+ * Copyright (C) 2009  Kristian Rietveld  <kris@gtk.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,6 +26,7 @@
 
 #include <gtk/gtk.h>
 #include <unistd.h>
+#include <math.h>
 
 #define VIEW_WIDTH 320
 #define VIEW_HEIGHT 240
@@ -37,6 +39,16 @@
  *   - Test that nothing happens if the row is fully visible.
  *   - The tests are dependent on the theme/font (size measurements,
  *     chosen paths).
+ *   - Convert to proper GTK+ coding style.
+ *   - Briefly test scrolling in tree stores as well.
+ *
+ * Important:
+ *   - For tests with "mixed height" models, you must ensure that
+ *     there are only two heights used in total and that the rows with
+ *     height A and B are strictly alternating.  The model creation
+ *     functions already do this for you, but take this into account
+ *     when you write a unit test that adds rows to such a created
+ *     model, you must follow this rule otherwise things will break.
  */
 
 
@@ -106,10 +118,15 @@ scroll_fixture_setup (ScrollFixture *fixture,
 	fixture->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
 	sw = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
+					GTK_POLICY_NEVER,
+					GTK_POLICY_ALWAYS);
 	gtk_container_add (GTK_CONTAINER (fixture->window), sw);
 
 	fixture->tree_view = gtk_tree_view_new_with_model (model);
 	g_object_unref (model);
+	gtk_scrolled_window_set_min_content_width (GTK_SCROLLED_WINDOW (sw), VIEW_WIDTH);
+	gtk_scrolled_window_set_min_content_height (GTK_SCROLLED_WINDOW (sw), VIEW_HEIGHT);
 	gtk_widget_set_size_request (fixture->tree_view, VIEW_WIDTH, VIEW_HEIGHT);
 
 	renderer = gtk_cell_renderer_text_new ();
@@ -175,6 +192,68 @@ scroll_fixture_single_setup (ScrollFixture *fixture,
 	scroll_fixture_setup (fixture, GTK_TREE_MODEL (store), test_data);
 }
 
+/* sets up a fixture with a tree store */
+static void
+scroll_fixture_tree_setup (ScrollFixture *fixture,
+			   gconstpointer   test_data)
+{
+	GtkTreeStore *store;
+	GtkTreeIter iter, child;
+	int i;
+
+	store = gtk_tree_store_new (1, G_TYPE_STRING);
+
+	gtk_tree_store_append (store, &iter, NULL);
+	gtk_tree_store_set (store, &iter, 0, "Root node", -1);
+
+	for (i = 0; i < 5; i++) {
+		gtk_tree_store_append (store, &child, &iter);
+		gtk_tree_store_set (store, &child, 0, "Child node", -1);
+	}
+
+	for (i = 0; i < 5; i++) {
+		gtk_tree_store_append (store, &iter, NULL);
+		gtk_tree_store_set (store, &iter, 0, "Other node", -1);
+	}
+
+	/* The teardown will also destroy the model */
+	scroll_fixture_setup (fixture, GTK_TREE_MODEL (store), test_data);
+}
+
+static void
+scroll_fixture_mixed_tree_setup (ScrollFixture *fixture,
+			         gconstpointer   test_data)
+{
+	GtkTreeStore *store;
+	GtkTreeIter iter, child;
+	int i;
+
+	store = gtk_tree_store_new (1, G_TYPE_STRING);
+
+	gtk_tree_store_append (store, &iter, NULL);
+	gtk_tree_store_set (store, &iter, 0, "Root\nnode", -1);
+
+	for (i = 0; i < 5; i++) {
+		gtk_tree_store_append (store, &child, &iter);
+		if (i % 2 != 0)
+			gtk_tree_store_set (store, &child, 0, "Child node", -1);
+		else
+			gtk_tree_store_set (store, &child,
+					    0, "Child\nnode", -1);
+	}
+
+	for (i = 0; i < 5; i++) {
+		gtk_tree_store_append (store, &iter, NULL);
+		if (i % 2 == 0)
+			gtk_tree_store_set (store, &iter, 0, "Other node", -1);
+		else
+			gtk_tree_store_set (store, &iter, 0, "Other\nnode", -1);
+	}
+
+	/* The teardown will also destroy the model */
+	scroll_fixture_setup (fixture, GTK_TREE_MODEL (store), test_data);
+}
+
 static void
 scroll_fixture_teardown (ScrollFixture *fixture,
 			 gconstpointer  test_data)
@@ -219,18 +298,18 @@ get_row_start_for_index (GtkTreeView *tree_view, int index)
 static enum Pos
 get_pos_from_path (GtkTreeView   *tree_view,
 		   GtkTreePath   *path,
-		   gint           row_height,
-		   GtkAdjustment *vadj)
+		   gdouble        row_height,
+		   GtkAdjustment *vadjustment)
 {
 	int row_start;
 
 	row_start = get_row_start_for_index (tree_view,
 					     gtk_tree_path_get_indices (path)[0]);
 
-	if (row_start + row_height < vadj->page_size)
+	if (row_start + row_height < gtk_adjustment_get_page_size (vadjustment))
 		return POS_TOP;
 
-	if (row_start >= vadj->upper - vadj->page_size)
+	if (row_start >= gtk_adjustment_get_upper (vadjustment) - gtk_adjustment_get_page_size (vadjustment))
 		return POS_BOTTOM;
 
 	return POS_CENTER;
@@ -241,11 +320,11 @@ test_position_with_align (GtkTreeView  *tree_view,
 			  enum Pos      pos,
 			  gint          row_y,
 			  gint          row_start,
-			  gint          row_height,
-			  gfloat        row_align)
+			  gdouble       row_height,
+			  gdouble       row_align)
 {
 	gboolean passed = TRUE;
-	GtkAdjustment *vadj = gtk_tree_view_get_vadjustment (tree_view);
+	GtkAdjustment *vadjustment = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (tree_view));
 
 	/* Switch on row-align: 0.0, 0.5, 1.0 */
 	switch ((int)(row_align * 2.)) {
@@ -259,7 +338,7 @@ test_position_with_align (GtkTreeView  *tree_view,
 			 */
 			if (row_y != 0)
 				passed = FALSE;
-			if (vadj->value != row_start)
+			if (gtk_adjustment_get_value (vadjustment) != row_start)
 				passed = FALSE;
 		} else {
 			/* The row can be anywhere at the last
@@ -267,7 +346,7 @@ test_position_with_align (GtkTreeView  *tree_view,
 			 *   - dy is set to the start of the
 			 *     last page.
 			 */
-			if (vadj->value != vadj->upper - vadj->page_size)
+			if (gtk_adjustment_get_value (vadjustment) != gtk_adjustment_get_upper (vadjustment) - gtk_adjustment_get_page_size (vadjustment))
 				passed = FALSE;
 		}
 		break;
@@ -275,23 +354,23 @@ test_position_with_align (GtkTreeView  *tree_view,
 	case 1:
 		/* 0.5 */
 		if (pos == POS_TOP
-		    && row_start < vadj->page_size / 2) {
+		    && row_start < gtk_adjustment_get_page_size (vadjustment) / 2) {
 			/* For the first half of the top view we can't
 			 * center the row in the view, instead we
 			 * show the first page.
 			 *   - dy should be zero
 			 */
-			if (vadj->value != 0)
+			if (gtk_adjustment_get_value (vadjustment) != 0)
 				passed = FALSE;
 		} else if (pos == POS_BOTTOM
-			   && row_start >= vadj->upper - vadj->page_size / 2) {
+			   && row_start >= gtk_adjustment_get_upper (vadjustment) - gtk_adjustment_get_page_size (vadjustment) / 2) {
 			/* For the last half of the bottom view we
 			 * can't center the row in the view, instead
 			 * we show the last page.
 			 *   - dy should be the start of the 
 			 *     last page.
 			 */
-			if (vadj->value != vadj->upper - vadj->page_size)
+			if (gtk_adjustment_get_value (vadjustment) != gtk_adjustment_get_upper (vadjustment) - gtk_adjustment_get_page_size (vadjustment))
 				passed = FALSE;
 		} else {
 			/* The row is located in the middle of
@@ -302,7 +381,8 @@ test_position_with_align (GtkTreeView  *tree_view,
 			 *      (ie. the row's center is at the
 			 *       center of the view).
 			 */
-			if (row_y != (int)(vadj->page_size / 2 - row_height / 2))
+			gdouble middle = gtk_adjustment_get_page_size (vadjustment) / 2 - row_height / 2;
+			if (row_y != ceil (middle) && row_y != floor (middle))
 				passed = FALSE;
 		}
 		break;
@@ -314,7 +394,7 @@ test_position_with_align (GtkTreeView  *tree_view,
 			 * first page of the tree view.
 			 *   - dy is zero.
 			 */
-			if (vadj->value != 0)
+			if (gtk_adjustment_get_value (vadjustment) != 0)
 				passed = FALSE;
 		} else if (pos == POS_CENTER || pos == POS_BOTTOM) {
 			/* The row is the last row visible in the
@@ -325,12 +405,12 @@ test_position_with_align (GtkTreeView  *tree_view,
 			 *     (ie we are not on the first page).
 			 *   - dy is greater than zero
 			 */
-			if (row_start < vadj->page_size
-			    && row_start + row_height < vadj->page_size)
+			if (row_start < gtk_adjustment_get_page_size (vadjustment)
+			    && row_start + row_height < gtk_adjustment_get_page_size (vadjustment))
 				passed = FALSE;
-			if (vadj->value <= 0)
+			if (gtk_adjustment_get_value (vadjustment) <= 0)
 				passed = FALSE;
-			if (row_y != vadj->page_size - row_height)
+			if (row_y != gtk_adjustment_get_page_size (vadjustment) - row_height)
 				passed = FALSE;
 		}
 		break;
@@ -341,17 +421,17 @@ test_position_with_align (GtkTreeView  *tree_view,
 
 static gboolean
 test_position_without_align (GtkTreeView *tree_view,
-			     gint         row_start,
-			     gint         row_height)
+			     gdouble      row_start,
+			     gdouble      row_height)
 {
-	GtkAdjustment *vadj = gtk_tree_view_get_vadjustment (tree_view);
+  GtkAdjustment *vadjustment = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (tree_view));
 
 	/* Without align the tree view does as less work as possible,
 	 * so basically we only have to check whether the row
 	 * is visible on the screen.
 	 */
-	if (vadj->value <= row_start
-	    && vadj->value + vadj->page_size >= row_start + row_height)
+	if (gtk_adjustment_get_value (vadjustment) <= row_start
+	    && gtk_adjustment_get_value (vadjustment) + gtk_adjustment_get_page_size (vadjustment) >= row_start + row_height)
 		return TRUE;
 
 	return FALSE;
@@ -361,8 +441,8 @@ static void
 test_position (GtkTreeView *tree_view,
 	       GtkTreePath *path,
 	       gboolean     use_align,
-	       gfloat       row_align,
-	       gfloat       col_align)
+	       gdouble      row_align,
+	       gdouble      col_align)
 {
 	gint pos;
 	gchar *path_str;
@@ -380,11 +460,12 @@ test_position (GtkTreeView *tree_view,
 	/* Ugh */
 	pos = get_pos_from_path (GTK_TREE_VIEW (tree_view),
 				 path, rect.height,
-			         gtk_tree_view_get_vadjustment (GTK_TREE_VIEW (tree_view)));
+			         gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (tree_view)));
 
 	/* This is only tested for during test_single() */
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view));
 	if (gtk_tree_model_iter_n_children (model, NULL) == 1) {
+                GtkAllocation allocation;
 		GtkTreePath *tmppath;
 
 		/* Test nothing is dangling at the bottom; read
@@ -392,7 +473,8 @@ test_position (GtkTreeView *tree_view,
 		 */
 
 		/* FIXME: hardcoded width */
-		if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (tree_view), 0, GTK_WIDGET (tree_view)->allocation.height - 30, &tmppath, NULL, NULL, NULL)) {
+                gtk_widget_get_allocation (GTK_WIDGET (tree_view), &allocation);
+		if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (tree_view), 0, allocation.height - 30, &tmppath, NULL, NULL, NULL)) {
 			g_assert_not_reached ();
 			gtk_tree_path_free (tmppath);
 		}
@@ -420,7 +502,7 @@ static void
 scroll (ScrollFixture *fixture,
 	GtkTreePath   *path,
 	gboolean       use_align,
-	gfloat         row_align)
+	gdouble        row_align)
 {
 	gtk_tree_view_set_cursor (GTK_TREE_VIEW (fixture->tree_view), path,
 				  NULL, FALSE);
@@ -486,7 +568,7 @@ static void
 scroll_after_realize (ScrollFixture *fixture,
 		      GtkTreePath   *path,
 		      gboolean       use_align,
-		      gfloat         row_align)
+		      gdouble        row_align)
 {
 	gtk_widget_show_all (fixture->window);
 
@@ -555,7 +637,7 @@ static void
 scroll_both_realize (ScrollFixture *fixture,
 		     GtkTreePath   *path,
 		     gboolean       use_align,
-		     gfloat         row_align)
+		     gdouble        row_align)
 {
 	GtkTreePath *end;
 
@@ -644,11 +726,21 @@ create_new_row (GtkListStore *store,
 			gtk_list_store_prepend (store, iter);
 			break;
 
+                case 3:
+			/* Add a row in the middle of the visible area */
+			gtk_list_store_insert (store, iter, 3);
+			break;
+
 		case 4:
 			/* Add a row in the middle of the visible area */
 			gtk_list_store_insert (store, iter, 4);
 			break;
 
+                case 5:
+			/* Add a row which is not completely visible */
+			gtk_list_store_insert (store, iter, 5);
+			break;
+
 		case 8:
 			/* Add a row which is not completely visible */
 			gtk_list_store_insert (store, iter, 8);
@@ -684,16 +776,18 @@ test_editable_position (GtkWidget   *tree_view,
 			GtkWidget   *editable,
 			GtkTreePath *cursor_path)
 {
+        GtkAllocation allocation;
 	GdkRectangle rect;
-	GtkAdjustment *vadj;
+	GtkAdjustment *vadjustment;
 
 	gtk_tree_view_get_background_area (GTK_TREE_VIEW (tree_view),
 					   cursor_path, NULL, &rect);
 
-	vadj = gtk_tree_view_get_vadjustment (GTK_TREE_VIEW (tree_view));
+	vadjustment = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (tree_view));
 
 	/* There are all in bin_window coordinates */
-	g_assert (editable->allocation.y == rect.y + ((rect.height - editable->allocation.height) / 2));
+        gtk_widget_get_allocation (editable, &allocation);
+        g_assert (allocation.y == rect.y + ((rect.height - allocation.height) / 2));
 }
 
 static void
@@ -728,7 +822,7 @@ scroll_new_row (ScrollFixture *fixture,
 
 	/* Set up a signal handler to acquire the editable widget */
 	column = gtk_tree_view_get_column (GTK_TREE_VIEW (fixture->tree_view), 0);
-	renderers = gtk_tree_view_column_get_cell_renderers (column);
+	renderers = gtk_cell_layout_get_cells (GTK_CELL_LAYOUT (column));
 
 	g_signal_connect (G_OBJECT (renderers->data), "editing-started",
 			  G_CALLBACK (scroll_new_row_editing_started),
@@ -752,6 +846,56 @@ scroll_new_row (ScrollFixture *fixture,
 	gtk_tree_path_free (scroll_path);
 }
 
+static void
+scroll_new_row_tree (ScrollFixture *fixture,
+		     gconstpointer  test_data)
+{
+	GtkTreeModel *model;
+	GtkAdjustment *vadjustment;
+	int i;
+
+	/* The goal of this test is to append new rows at the end of a tree
+	 * store and immediately scroll to them.  If there is a parent
+	 * node with a couple of childs in the "area above" to explore,
+	 * this used to lead to unexpected results due to a bug.
+	 *
+	 * This issue has been reported by Miroslav Rajcic on
+	 * gtk-app-devel-list:
+	 * http://mail.gnome.org/archives/gtk-app-devel-list/2008-December/msg00068.html
+	 */
+
+	gtk_widget_show_all (fixture->window);
+
+	gtk_tree_view_expand_all (GTK_TREE_VIEW (fixture->tree_view));
+
+	while (gtk_events_pending ())
+		gtk_main_iteration ();
+
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (fixture->tree_view));
+	vadjustment = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (fixture->tree_view));
+
+	for (i = 0; i < 5; i++) {
+		GtkTreeIter scroll_iter;
+		GtkTreePath *scroll_path;
+
+		gtk_tree_store_append (GTK_TREE_STORE (model), &scroll_iter,
+				       NULL);
+		gtk_tree_store_set (GTK_TREE_STORE (model), &scroll_iter,
+				    0, "New node", -1);
+
+		scroll_path = gtk_tree_model_get_path (model, &scroll_iter);
+		gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (fixture->tree_view),
+					      scroll_path, NULL, FALSE, 0.0, 0.0);
+		gtk_tree_path_free (scroll_path);
+
+		while (gtk_events_pending ())
+			gtk_main_iteration ();
+
+		/* Test position, the scroll bar must be at the end */
+		g_assert (gtk_adjustment_get_value (vadjustment) == gtk_adjustment_get_upper (vadjustment) - gtk_adjustment_get_page_size (vadjustment));
+	}
+}
+
 /* Test for GNOME bugzilla bug 359231; tests "recovery when removing a bunch of
  * rows at the bottom.
  */
@@ -761,7 +905,7 @@ test_bug316689 (ScrollFixture *fixture,
 {
 	GtkTreeIter iter;
 	GtkTreePath *path;
-	GtkAdjustment *vadj;
+	GtkAdjustment *vadjustment;
 	GtkTreeModel *model;
 
 	/* The aim of this test is to scroll to the bottom of a TreeView,
@@ -791,10 +935,10 @@ test_bug316689 (ScrollFixture *fixture,
 	while (gtk_events_pending ())
 		gtk_main_iteration ();
 
-	vadj = gtk_tree_view_get_vadjustment (GTK_TREE_VIEW (fixture->tree_view));
+	vadjustment = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (fixture->tree_view));
 
-	g_assert (vadj->value + vadj->page_size <= vadj->upper);
-	g_assert (vadj->value == vadj->upper - vadj->page_size);
+	g_assert (gtk_adjustment_get_value (vadjustment) + gtk_adjustment_get_page_size (vadjustment) <= gtk_adjustment_get_upper (vadjustment));
+	g_assert (gtk_adjustment_get_value (vadjustment) == gtk_adjustment_get_upper (vadjustment) - gtk_adjustment_get_page_size (vadjustment));
 }
 
 
@@ -861,6 +1005,212 @@ test_bug359231 (void)
 	scroll_fixture_teardown (fixture, NULL);
 }
 
+/* Test for GNOME bugzilla bug 93584.  We add 150 rows to an existing
+ * small model, and scroll to one of these with alignment.
+ */
+static void
+test_bug93584 (ScrollFixture *fixture,
+	       gconstpointer  test_data)
+{
+	int row, i;
+	GtkTreeStore *store;
+	GtkTreePath *path;
+
+	g_test_bug ("93584");
+
+	/* Mimic state as in original test case */
+	g_signal_connect (G_OBJECT (fixture->tree_view), "realize",
+			  G_CALLBACK (gtk_tree_view_expand_all), NULL);
+	gtk_widget_show_all (fixture->window);
+
+	store = GTK_TREE_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (fixture->tree_view)));
+
+	/* Add 150 rows */
+	for (i = 0; i < 150; i++) {
+		GtkTreeIter iter;
+
+		gtk_tree_store_append (store, &iter, NULL);
+		gtk_tree_store_set (store, &iter, 0, "Row", -1);
+	}
+
+	row = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (store), NULL);
+	row -= 20;
+
+	path = gtk_tree_path_new_from_indices (row, -1);
+	scroll (fixture, path, TRUE, 0.5);
+	gtk_tree_path_free (path);
+}
+
+/* GNOME bugzilla bug 111500.  Expand a row and immediately scroll
+ * to its first child.  Make sure that expansion happens in currently
+ * invisible area.
+ */
+static void
+test_bug111500 (ScrollFixture *fixture,
+		gconstpointer  test_data)
+{
+	int i, len;
+	GtkTreeStore *store;
+	GtkTreeIter parent;
+	GtkTreePath *path;
+
+	g_test_bug ("111500");
+
+	gtk_widget_show_all (fixture->window);
+
+	/* Make sure all events have been processed and the window
+	 * is visible.
+	 */
+	while (gtk_events_pending ())
+		gtk_main_iteration ();
+
+	/* Further prepare model */
+	store = GTK_TREE_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (fixture->tree_view)));
+
+	for (i = 0; i < 15; i++) {
+		GtkTreeIter iter;
+
+		gtk_tree_store_append (store, &iter, NULL);
+		gtk_tree_store_set (store, &iter, 0, "Other node", -1);
+	}
+
+	len = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (store), NULL);
+	gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (store), &parent,
+				       NULL, len - 1);
+
+	for (i = 0; i < 5; i++) {
+		GtkTreeIter iter;
+
+		gtk_tree_store_append (store, &iter, &parent);
+		gtk_tree_store_set (store, &iter, 0, "Row", -1);
+	}
+
+	path = gtk_tree_path_new_from_indices (len - 1, -1);
+	gtk_tree_view_expand_row (GTK_TREE_VIEW (fixture->tree_view),
+				  path, FALSE);
+
+	gtk_tree_path_down (path);
+
+	scroll (fixture, path, TRUE, 0.5);
+	gtk_tree_path_free (path);
+}
+
+static void
+test_bug111500_mixed (ScrollFixture *fixture,
+		      gconstpointer  test_data)
+{
+	int i, len;
+	GtkTreeStore *store;
+	GtkTreeIter parent;
+	GtkTreePath *path;
+
+	g_test_bug ("111500");
+
+	gtk_widget_show_all (fixture->window);
+
+	/* Make sure all events have been processed and the window
+	 * is visible.
+	 */
+	while (gtk_events_pending ())
+		gtk_main_iteration ();
+
+	/* Further prepare model */
+	store = GTK_TREE_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (fixture->tree_view)));
+
+	for (i = 0; i < 15; i++) {
+		GtkTreeIter iter;
+
+		gtk_tree_store_append (store, &iter, NULL);
+		if (i % 2 == 0)
+			gtk_tree_store_set (store, &iter, 0, "Other node", -1);
+		else
+			gtk_tree_store_set (store, &iter, 0, "Other\nnode", -1);
+	}
+
+	len = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (store), NULL);
+	gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (store), &parent,
+				       NULL, len - 1);
+
+	for (i = 0; i < 5; i++) {
+		GtkTreeIter iter;
+
+		gtk_tree_store_append (store, &iter, &parent);
+		if (i % 2 != 0)
+			gtk_tree_store_set (store, &iter, 0, "Row", -1);
+		else
+			gtk_tree_store_set (store, &iter, 0, "Row\nRow", -1);
+	}
+
+	path = gtk_tree_path_new_from_indices (len - 1, -1);
+	gtk_tree_view_expand_row (GTK_TREE_VIEW (fixture->tree_view),
+				  path, FALSE);
+
+	gtk_tree_path_down (path);
+
+	scroll (fixture, path, TRUE, 0.5);
+	gtk_tree_path_free (path);
+}
+
+/* Test for GNOME bugzilla bug 163214.  Invalidate a couple of rows,
+ * then scroll to one of these.
+ */
+static void
+test_bug163214 (ScrollFixture *fixture,
+		gconstpointer  test_data)
+{
+	int i;
+	GtkListStore *store;
+	GtkTreePath *path;
+
+	g_test_bug ("163214");
+
+	gtk_widget_show_all (fixture->window);
+
+	/* Make sure all events have been processed and the window
+	 * is visible.
+	 */
+	while (gtk_events_pending ())
+		gtk_main_iteration ();
+
+	store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (fixture->tree_view)));
+
+	/* Invalidate a page of rows */
+	for (i = 100; i < 110; i++) {
+		GtkTreeIter iter;
+
+		gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (store), &iter,
+					       NULL, i);
+		gtk_list_store_set (store, &iter, 0, "Row", -1);
+	}
+
+	/* Then scroll to that page. */
+	path = gtk_tree_path_new_from_indices (105, -1);
+	scroll (fixture, path, TRUE, 0.5);
+	gtk_tree_path_free (path);
+
+	/* Make sure all events have been processed and the window
+	 * is visible.
+	 */
+	while (gtk_events_pending ())
+		gtk_main_iteration ();
+
+	store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (fixture->tree_view)));
+
+	/* Invalidate a page of rows */
+	for (i = 300; i < 310; i++) {
+		GtkTreeIter iter;
+
+		gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (store), &iter,
+					       NULL, i);
+		gtk_list_store_set (store, &iter, 0, "Row", -1);
+	}
+
+	/* Then scroll to the first row */
+	path = gtk_tree_path_new_from_indices (0, -1);
+	scroll (fixture, path, TRUE, 0.5);
+	gtk_tree_path_free (path);
+}
+
 /* Infrastructure for automatically adding tests */
 enum
 {
@@ -874,10 +1224,10 @@ test_type_string (int test_type)
 {
 	switch (test_type) {
 		case BEFORE:
-			return "before";
+			return "before-realize";
 
 		case AFTER:
-			return "after";
+			return "after-realize";
 
 		case BOTH:
 			return "both";
@@ -888,7 +1238,7 @@ test_type_string (int test_type)
 
 static char *
 align_string (gboolean use_align,
-	      gfloat   row_align)
+	      gdouble  row_align)
 {
 	char *ret;
 
@@ -904,7 +1254,7 @@ add_test (const char *path,
 	  gboolean    mixed,
 	  int         test_type,
 	  gboolean    use_align,
-	  gfloat      row_align,
+	  gdouble     row_align,
 	  void (* setup) (ScrollFixture *, gconstpointer),
 	  void (* scroll_func) (ScrollFixture *, gconstpointer))
 {
@@ -913,7 +1263,7 @@ add_test (const char *path,
 
 	align = align_string (use_align, row_align);
 
-	test_path = g_strdup_printf ("/treeview/scrolling/%s-%s-path-%s-%s",
+	test_path = g_strdup_printf ("/TreeView/scrolling/%s/%s-height/path-%s-%s",
 				     test_type_string (test_type),
 				     mixed ? "mixed" : "constant",
 				     path, align);
@@ -929,7 +1279,7 @@ static void
 add_tests (gboolean mixed,
 	   int test_type,
 	   gboolean use_align,
-	   gfloat row_align,
+	   gdouble  row_align,
 	   void (*scroll_func) (ScrollFixture *, gconstpointer))
 {
 	void (* setup) (ScrollFixture *, gconstpointer);
@@ -1013,42 +1363,46 @@ main (int argc, char **argv)
 	}
 
 	/* Test different alignments in view with single row */
-	g_test_add ("/treeview/scrolling/single-no-align", ScrollFixture, "0",
+	g_test_add ("/TreeView/scrolling/single-row/no-align",
+		    ScrollFixture, "0",
 		    scroll_fixture_single_setup,
 		    scroll_no_align,
 		    scroll_fixture_teardown);
-	g_test_add ("/treeview/scrolling/single-align-0.0", ScrollFixture, "0",
+	g_test_add ("/TreeView/scrolling/single-row/align-0.0",
+		    ScrollFixture, "0",
 		    scroll_fixture_single_setup,
 		    scroll_align_0_0,
 		    scroll_fixture_teardown);
-	g_test_add ("/treeview/scrolling/single-align-0.5", ScrollFixture, "0",
+	g_test_add ("/TreeView/scrolling/single-row/align-0.5",
+		    ScrollFixture, "0",
 		    scroll_fixture_single_setup,
 		    scroll_align_0_5,
 		    scroll_fixture_teardown);
-	g_test_add ("/treeview/scrolling/single-align-1.0", ScrollFixture, "0",
+	g_test_add ("/TreeView/scrolling/single-row/align-1.0",
+		    ScrollFixture, "0",
 		    scroll_fixture_single_setup,
 		    scroll_align_1_0,
 		    scroll_fixture_teardown);
 
 	/* Test scrolling in a very large model; also very slow */
 	if (g_test_slow ()) {
-		g_test_add ("/treeview/scrolling/constant-big-middle-no-align",
+		g_test_add ("/TreeView/scrolling/large-model/constant-height/middle-no-align",
 			    ScrollFixture, "50000",
 			    scroll_fixture_constant_big_setup,
 			    scroll_no_align,
 			    scroll_fixture_teardown);
-		g_test_add ("/treeview/scrolling/constant-big-end-no-align",
+		g_test_add ("/TreeView/scrolling/large-model/constant-height/end-no-align",
 			    ScrollFixture, "99999",
 			    scroll_fixture_constant_big_setup,
 			    scroll_no_align,
 			    scroll_fixture_teardown);
 
-		g_test_add ("/treeview/scrolling/mixed-big-middle-no-align",
+		g_test_add ("/TreeView/scrolling/large-model/mixed-height/middle-no-align",
 			    ScrollFixture, "50000",
 			    scroll_fixture_mixed_big_setup,
 			    scroll_no_align,
 			    scroll_fixture_teardown);
-		g_test_add ("/treeview/scrolling/mixed-big-end-no-align",
+		g_test_add ("/TreeView/scrolling/large-model/mixed-height/end-no-align",
 			    ScrollFixture, "99999",
 			    scroll_fixture_mixed_big_setup,
 			    scroll_no_align,
@@ -1056,12 +1410,12 @@ main (int argc, char **argv)
 	}
 
 	/* Test scrolling to a newly created row */
-	g_test_add ("/treeview/scrolling/new-row-path-0", ScrollFixture,
+	g_test_add ("/TreeView/scrolling/new-row/path-0", ScrollFixture,
 		    GINT_TO_POINTER (0),
 		    scroll_fixture_constant_setup,
 		    scroll_new_row,
 		    scroll_fixture_teardown);
-	g_test_add ("/treeview/scrolling/new-row-path-4", ScrollFixture,
+	g_test_add ("/TreeView/scrolling/new-row/path-4", ScrollFixture,
 		    GINT_TO_POINTER (4),
 		    scroll_fixture_constant_setup,
 		    scroll_new_row,
@@ -1070,27 +1424,88 @@ main (int argc, char **argv)
 	 * based on my font setting of "Vera Sans 11" and
 	 * the separators set to 0.  (This should be made dynamic; FIXME).
 	 */
-	g_test_add ("/treeview/scrolling/new-row-path-8", ScrollFixture,
+	g_test_add ("/TreeView/scrolling/new-row/path-8", ScrollFixture,
 		    GINT_TO_POINTER (8),
 		    scroll_fixture_constant_setup,
 		    scroll_new_row,
 		    scroll_fixture_teardown);
-	g_test_add ("/treeview/scrolling/new-row-path-500", ScrollFixture,
+	g_test_add ("/TreeView/scrolling/new-row/path-500", ScrollFixture,
 		    GINT_TO_POINTER (500),
 		    scroll_fixture_constant_setup,
 		    scroll_new_row,
 		    scroll_fixture_teardown);
-	g_test_add ("/treeview/scrolling/new-row-path-999", ScrollFixture,
+	g_test_add ("/TreeView/scrolling/new-row/path-999", ScrollFixture,
 		    GINT_TO_POINTER (999),
 		    scroll_fixture_constant_setup,
 		    scroll_new_row,
 		    scroll_fixture_teardown);
 
+	g_test_add ("/TreeView/scrolling/new-row/tree", ScrollFixture,
+		    NULL,
+		    scroll_fixture_tree_setup,
+		    scroll_new_row_tree,
+		    scroll_fixture_teardown);
+
+	/* Test scrolling to a newly created row, in a mixed height model */
+	g_test_add ("/TreeView/scrolling/new-row-mixed/path-0", ScrollFixture,
+		    GINT_TO_POINTER (0),
+		    scroll_fixture_mixed_setup,
+		    scroll_new_row,
+		    scroll_fixture_teardown);
+	g_test_add ("/TreeView/scrolling/new-row-mixed/path-3", ScrollFixture,
+		    GINT_TO_POINTER (3),
+		    scroll_fixture_mixed_setup,
+		    scroll_new_row,
+		    scroll_fixture_teardown);
+	/* We scroll to 8 to test a partial visible row.  The 8 is
+	 * based on my font setting of "Vera Sans 11" and
+	 * the separators set to 0.  (This should be made dynamic; FIXME).
+	 */
+	g_test_add ("/TreeView/scrolling/new-row-mixed/path-5", ScrollFixture,
+		    GINT_TO_POINTER (5),
+		    scroll_fixture_mixed_setup,
+		    scroll_new_row,
+		    scroll_fixture_teardown);
+	g_test_add ("/TreeView/scrolling/new-row-mixed/path-500", ScrollFixture,
+		    GINT_TO_POINTER (500),
+		    scroll_fixture_mixed_setup,
+		    scroll_new_row,
+		    scroll_fixture_teardown);
+	g_test_add ("/TreeView/scrolling/new-row-mixed/path-999", ScrollFixture,
+		    GINT_TO_POINTER (999),
+		    scroll_fixture_mixed_setup,
+		    scroll_new_row,
+		    scroll_fixture_teardown);
+
+	g_test_add ("/TreeView/scrolling/new-row-mixed/tree", ScrollFixture,
+		    NULL,
+		    scroll_fixture_mixed_tree_setup,
+		    scroll_new_row_tree,
+		    scroll_fixture_teardown);
+
 	/* Misc. tests */
-	g_test_add ("/treeview/scrolling/bug-316689", ScrollFixture, NULL,
+	g_test_add ("/TreeView/scrolling/specific/bug-316689",
+			ScrollFixture, NULL,
 		    scroll_fixture_constant_setup, test_bug316689,
 		    scroll_fixture_teardown);
-	g_test_add_func ("/treeview/scrolling/bug-359231", test_bug359231);
+	g_test_add_func ("/TreeView/scrolling/specific/bug-359231",
+			test_bug359231);
+	g_test_add ("/TreeView/scrolling/specific/bug-93584",
+		    ScrollFixture, NULL,
+		    scroll_fixture_tree_setup, test_bug93584,
+		    scroll_fixture_teardown);
+	g_test_add ("/TreeView/scrolling/specific/bug-111500",
+		    ScrollFixture, NULL,
+		    scroll_fixture_tree_setup, test_bug111500,
+		    scroll_fixture_teardown);
+	g_test_add ("/TreeView/scrolling/specific/bug-111500-mixed",
+		    ScrollFixture, NULL,
+		    scroll_fixture_mixed_tree_setup, test_bug111500_mixed,
+		    scroll_fixture_teardown);
+	g_test_add ("/TreeView/scrolling/specific/bug-163214",
+		    ScrollFixture, NULL,
+		    scroll_fixture_constant_setup, test_bug163214,
+		    scroll_fixture_teardown);
 
 	return g_test_run ();
 }
