@@ -216,9 +216,7 @@ setup_buffer (GtkTextView  *view,
 {
   GtkTextBuffer *buffer;
 
-  buffer = view->buffer;
-  if (buffer == NULL)
-    return;
+  buffer = gtk_text_view_get_buffer (view);
 
   if (gail_view->textutil)
     g_object_unref (gail_view->textutil);
@@ -303,7 +301,7 @@ gail_text_view_ref_state_set (AtkObject *accessible)
   GtkWidget *widget;
 
   state_set = ATK_OBJECT_CLASS (gail_text_view_parent_class)->ref_state_set (accessible);
-  widget = GTK_ACCESSIBLE (accessible)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (accessible));
 
   if (widget == NULL)
     return state_set;
@@ -351,13 +349,13 @@ gail_text_view_get_text (AtkText *text,
   GtkTextIter start, end;
   GtkWidget *widget;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return NULL;
 
   view = GTK_TEXT_VIEW (widget);
-  buffer = view->buffer;
+  buffer = gtk_text_view_get_buffer (view);
   gtk_text_buffer_get_iter_at_offset (buffer, &start, start_offset);
   gtk_text_buffer_get_iter_at_offset (buffer, &end, end_offset);
 
@@ -373,7 +371,7 @@ gail_text_view_get_text_after_offset (AtkText         *text,
 {
   GtkWidget *widget;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return NULL;
@@ -392,7 +390,7 @@ gail_text_view_get_text_at_offset (AtkText         *text,
 {
   GtkWidget *widget;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return NULL;
@@ -411,7 +409,7 @@ gail_text_view_get_text_before_offset (AtkText         *text,
 {
   GtkWidget *widget;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return NULL;
@@ -431,7 +429,7 @@ gail_text_view_get_character_at_offset (AtkText *text,
   gchar *string;
   gunichar unichar;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     return '\0';
 
@@ -455,13 +453,13 @@ gail_text_view_get_character_count (AtkText *text)
   GtkTextBuffer *buffer;
   GtkWidget *widget;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return 0;
 
   view = GTK_TEXT_VIEW (widget);
-  buffer = view->buffer;
+  buffer = gtk_text_view_get_buffer (view);
   return gtk_text_buffer_get_char_count (buffer);
 }
 
@@ -471,13 +469,13 @@ gail_text_view_get_caret_offset (AtkText *text)
   GtkTextView *view;
   GtkWidget *widget;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return 0;
 
   view = GTK_TEXT_VIEW (widget);
-  return get_insert_offset (view->buffer);
+  return get_insert_offset (gtk_text_view_get_buffer (view));
 }
 
 static gboolean
@@ -489,13 +487,13 @@ gail_text_view_set_caret_offset (AtkText *text,
   GtkTextBuffer *buffer;
   GtkTextIter pos_itr;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return FALSE;
 
   view = GTK_TEXT_VIEW (widget);
-  buffer = view->buffer;
+  buffer = gtk_text_view_get_buffer (view);
 
   gtk_text_buffer_get_iter_at_offset (buffer,  &pos_itr, offset);
   gtk_text_buffer_place_cursor (buffer, &pos_itr);
@@ -517,13 +515,13 @@ gail_text_view_get_offset_at_point (AtkText      *text,
   GdkWindow *window;
   GdkRectangle rect;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return -1;
 
   view = GTK_TEXT_VIEW (widget);
-  buffer = view->buffer;
+  buffer = gtk_text_view_get_buffer (view);
 
   window = gtk_text_view_get_window (view, GTK_TEXT_WINDOW_WIDGET);
   gdk_window_get_origin (window, &x_widget, &y_widget);
@@ -581,13 +579,13 @@ gail_text_view_get_character_extents (AtkText      *text,
   GdkWindow *window;
   gint x_widget, y_widget, x_window, y_window;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return;
 
   view = GTK_TEXT_VIEW (widget);
-  buffer = view->buffer;
+  buffer = gtk_text_view_get_buffer (view);
   gtk_text_buffer_get_iter_at_offset (buffer, &iter, offset);
   gtk_text_view_get_iter_location (view, &iter, &rectangle);
 
@@ -627,19 +625,31 @@ gail_text_view_get_run_attributes (AtkText *text,
                                    gint    *end_offset)
 {
   GtkTextView *view;
+  GtkTextBuffer *buffer;
   GtkWidget *widget;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return NULL;
 
   view = GTK_TEXT_VIEW (widget);
+  buffer = gtk_text_view_get_buffer (view);
 
-  return gail_misc_buffer_get_run_attributes (view->buffer, offset, 
+  return gail_misc_buffer_get_run_attributes (buffer, offset, 
                                               start_offset, end_offset);
 }
 
+static AtkAttributeSet*
+add_text_attribute (AtkAttributeSet *attrib_set, AtkTextAttribute attr, gint i)
+{
+  const gchar *value;
+
+  value = atk_text_attribute_get_value (attr, i);
+
+  return gail_misc_add_attribute (attrib_set, i, g_strdup (value));
+}
+
 static AtkAttributeSet*
 gail_text_view_get_default_attributes (AtkText *text)
 {
@@ -648,8 +658,9 @@ gail_text_view_get_default_attributes (AtkText *text)
   GtkTextAttributes *text_attrs;
   AtkAttributeSet *attrib_set = NULL;
   PangoFontDescription *font;
+  gchar *value;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return NULL;
@@ -661,96 +672,75 @@ gail_text_view_get_default_attributes (AtkText *text)
 
   if (font)
     {
-      attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                              ATK_TEXT_ATTR_STYLE);
+      attrib_set = add_text_attribute (attrib_set, ATK_TEXT_ATTR_STYLE,
+                                       pango_font_description_get_style (font));
 
-      attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                              ATK_TEXT_ATTR_VARIANT);
+      attrib_set = add_text_attribute (attrib_set, ATK_TEXT_ATTR_VARIANT,
+                                       pango_font_description_get_variant (font));
 
-      attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                              ATK_TEXT_ATTR_STRETCH);
+      attrib_set = add_text_attribute (attrib_set, ATK_TEXT_ATTR_STRETCH,
+                                       pango_font_description_get_stretch (font));
+
+      value = g_strdup (pango_font_description_get_family (font));
+      attrib_set = gail_misc_add_attribute (attrib_set, ATK_TEXT_ATTR_FAMILY_NAME, value);
+
+      value = g_strdup_printf ("%d", pango_font_description_get_weight (font));
+      attrib_set = gail_misc_add_attribute (attrib_set, ATK_TEXT_ATTR_WEIGHT, value);
+
+      value = g_strdup_printf ("%i", pango_font_description_get_size (font) / PANGO_SCALE);
+      attrib_set = gail_misc_add_attribute (attrib_set, ATK_TEXT_ATTR_SIZE, value);
     }
 
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_JUSTIFICATION);
+  attrib_set = add_text_attribute (attrib_set, ATK_TEXT_ATTR_JUSTIFICATION, text_attrs->justification);
+  attrib_set = add_text_attribute (attrib_set, ATK_TEXT_ATTR_DIRECTION, text_attrs->direction);
+  attrib_set = add_text_attribute (attrib_set, ATK_TEXT_ATTR_WRAP_MODE, text_attrs->wrap_mode);
+  attrib_set = add_text_attribute (attrib_set, ATK_TEXT_ATTR_EDITABLE, text_attrs->editable);
+  attrib_set = add_text_attribute (attrib_set, ATK_TEXT_ATTR_INVISIBLE, text_attrs->invisible);
+  attrib_set = add_text_attribute (attrib_set, ATK_TEXT_ATTR_BG_FULL_HEIGHT, text_attrs->bg_full_height);
 
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_DIRECTION);
+  attrib_set = add_text_attribute (attrib_set, ATK_TEXT_ATTR_STRIKETHROUGH,
+                                   text_attrs->appearance.strikethrough);
+  attrib_set = add_text_attribute (attrib_set, ATK_TEXT_ATTR_UNDERLINE,
+                                   text_attrs->appearance.underline);
 
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_WRAP_MODE);
+  value = g_strdup_printf ("%u,%u,%u",
+                           text_attrs->appearance.bg_color.red,
+                           text_attrs->appearance.bg_color.green,
+                           text_attrs->appearance.bg_color.blue);
+  attrib_set = gail_misc_add_attribute (attrib_set, ATK_TEXT_ATTR_BG_COLOR, value);
 
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_FG_STIPPLE);
+  value = g_strdup_printf ("%u,%u,%u",
+                           text_attrs->appearance.fg_color.red,
+                           text_attrs->appearance.fg_color.green,
+                           text_attrs->appearance.fg_color.blue);
+  attrib_set = gail_misc_add_attribute (attrib_set, ATK_TEXT_ATTR_FG_COLOR, value);
 
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_BG_STIPPLE);
+  value = g_strdup_printf ("%g", text_attrs->font_scale);
+  attrib_set = gail_misc_add_attribute (attrib_set, ATK_TEXT_ATTR_SCALE, value);
 
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_FG_COLOR);
+  value = g_strdup ((gchar *)(text_attrs->language));
+  attrib_set = gail_misc_add_attribute (attrib_set, ATK_TEXT_ATTR_LANGUAGE, value);
 
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_BG_COLOR);
+  value = g_strdup_printf ("%i", text_attrs->appearance.rise);
+  attrib_set = gail_misc_add_attribute (attrib_set, ATK_TEXT_ATTR_RISE, value);
 
-  if (font)
-    {
-      attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                              ATK_TEXT_ATTR_FAMILY_NAME);
-    }
+  value = g_strdup_printf ("%i", text_attrs->pixels_inside_wrap);
+  attrib_set = gail_misc_add_attribute (attrib_set, ATK_TEXT_ATTR_PIXELS_INSIDE_WRAP, value);
 
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_LANGUAGE);
+  value = g_strdup_printf ("%i", text_attrs->pixels_below_lines);
+  attrib_set = gail_misc_add_attribute (attrib_set, ATK_TEXT_ATTR_PIXELS_BELOW_LINES, value);
 
-  if (font)
-    {
-      attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                              ATK_TEXT_ATTR_WEIGHT);
-    }
+  value = g_strdup_printf ("%i", text_attrs->pixels_above_lines);
+  attrib_set = gail_misc_add_attribute (attrib_set, ATK_TEXT_ATTR_PIXELS_ABOVE_LINES, value);
 
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_SCALE);
+  value = g_strdup_printf ("%i", text_attrs->indent);
+  attrib_set = gail_misc_add_attribute (attrib_set, ATK_TEXT_ATTR_INDENT, value);
 
-  if (font)
-    {
-      attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                              ATK_TEXT_ATTR_SIZE);
-    }
+  value = g_strdup_printf ("%i", text_attrs->left_margin);
+  attrib_set = gail_misc_add_attribute (attrib_set, ATK_TEXT_ATTR_LEFT_MARGIN, value);
 
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_STRIKETHROUGH);
-
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_UNDERLINE);
-
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_RISE);
-
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_BG_FULL_HEIGHT);
-
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_PIXELS_INSIDE_WRAP);
-
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                         ATK_TEXT_ATTR_PIXELS_BELOW_LINES);
-
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_PIXELS_ABOVE_LINES);
-
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_EDITABLE);
-    
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_INVISIBLE);
-
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_INDENT);
-
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_RIGHT_MARGIN);
-
-  attrib_set = gail_misc_add_to_attr_set (attrib_set, text_attrs, 
-                                          ATK_TEXT_ATTR_LEFT_MARGIN);
+  value = g_strdup_printf ("%i", text_attrs->right_margin);
+  attrib_set = gail_misc_add_attribute (attrib_set, ATK_TEXT_ATTR_RIGHT_MARGIN, value);
 
   gtk_text_attributes_unref (text_attrs);
   return attrib_set;
@@ -765,13 +755,13 @@ gail_text_view_get_n_selections (AtkText *text)
   GtkTextIter start, end;
   gint select_start, select_end;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return -1;
 
   view = GTK_TEXT_VIEW (widget);
-  buffer = view->buffer;
+  buffer = gtk_text_view_get_buffer (view);
 
   gtk_text_buffer_get_selection_bounds (buffer, &start, &end);
   select_start = gtk_text_iter_get_offset (&start);
@@ -794,7 +784,7 @@ gail_text_view_get_selection (AtkText *text,
   GtkTextBuffer *buffer;
   GtkTextIter start, end;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return NULL;
@@ -806,7 +796,7 @@ gail_text_view_get_selection (AtkText *text,
      return NULL;
 
   view = GTK_TEXT_VIEW (widget);
-  buffer = view->buffer;
+  buffer = gtk_text_view_get_buffer (view);
 
   gtk_text_buffer_get_selection_bounds (buffer, &start, &end);
   *start_pos = gtk_text_iter_get_offset (&start);
@@ -830,13 +820,13 @@ gail_text_view_add_selection (AtkText *text,
   GtkTextIter start, end;
   gint select_start, select_end;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return FALSE;
 
   view = GTK_TEXT_VIEW (widget);
-  buffer = view->buffer;
+  buffer = gtk_text_view_get_buffer (view);
 
   gtk_text_buffer_get_selection_bounds (buffer, &start, &end);
   select_start = gtk_text_iter_get_offset (&start);
@@ -869,7 +859,7 @@ gail_text_view_remove_selection (AtkText *text,
   GtkTextIter start, end;
   gint select_start, select_end;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return FALSE;
@@ -878,7 +868,7 @@ gail_text_view_remove_selection (AtkText *text,
      return FALSE;
 
   view = GTK_TEXT_VIEW (widget);
-  buffer = view->buffer;
+  buffer = gtk_text_view_get_buffer (view);
 
   gtk_text_buffer_get_selection_bounds(buffer, &start, &end);
   select_start = gtk_text_iter_get_offset(&start);
@@ -911,7 +901,7 @@ gail_text_view_set_selection (AtkText *text,
   GtkTextIter start, end;
   gint select_start, select_end;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
   {
     /* State is defunct */
@@ -925,7 +915,7 @@ gail_text_view_set_selection (AtkText *text,
      return FALSE;
 
   view = GTK_TEXT_VIEW (widget);
-  buffer = view->buffer;
+  buffer = gtk_text_view_get_buffer (view);
 
   gtk_text_buffer_get_selection_bounds(buffer, &start, &end);
   select_start = gtk_text_iter_get_offset(&start);
@@ -974,7 +964,7 @@ gail_text_view_set_run_attributes (AtkEditableText *text,
   gchar** RGB_vals;
   GSList *l;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return FALSE;
@@ -983,7 +973,7 @@ gail_text_view_set_run_attributes (AtkEditableText *text,
   if (!gtk_text_view_get_editable (view))
     return FALSE;
 
-  buffer = view->buffer;
+  buffer = gtk_text_view_get_buffer (view);
 
   if (attrib_set == NULL)
     return FALSE;
@@ -1186,7 +1176,7 @@ gail_text_view_set_text_contents (AtkEditableText *text,
   GtkWidget *widget;
   GtkTextBuffer *buffer;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return;
@@ -1194,7 +1184,7 @@ gail_text_view_set_text_contents (AtkEditableText *text,
   view = GTK_TEXT_VIEW (widget);
   if (!gtk_text_view_get_editable (view))
     return;
-  buffer = view->buffer;
+  buffer = gtk_text_view_get_buffer (view);
 
   /* The -1 indicates that the input string must be null-terminated */
   gtk_text_buffer_set_text (buffer, string, -1);
@@ -1211,7 +1201,7 @@ gail_text_view_insert_text (AtkEditableText *text,
   GtkTextBuffer *buffer;
   GtkTextIter pos_itr;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return;
@@ -1219,7 +1209,7 @@ gail_text_view_insert_text (AtkEditableText *text,
   view = GTK_TEXT_VIEW (widget);
   if (!gtk_text_view_get_editable (view))
     return;
-  buffer = view->buffer;
+  buffer = gtk_text_view_get_buffer (view);
 
   gtk_text_buffer_get_iter_at_offset (buffer, &pos_itr, *position);
   gtk_text_buffer_insert (buffer, &pos_itr, string, length);
@@ -1237,13 +1227,13 @@ gail_text_view_copy_text   (AtkEditableText *text,
   gchar *str;
   GtkClipboard *clipboard;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return;
 
   view = GTK_TEXT_VIEW (widget);
-  buffer = view->buffer;
+  buffer = gtk_text_view_get_buffer (view);
 
   gtk_text_buffer_get_iter_at_offset (buffer, &start, start_pos);
   gtk_text_buffer_get_iter_at_offset (buffer, &end, end_pos);
@@ -1265,7 +1255,7 @@ gail_text_view_cut_text (AtkEditableText *text,
   gchar *str;
   GtkClipboard *clipboard;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return;
@@ -1273,7 +1263,7 @@ gail_text_view_cut_text (AtkEditableText *text,
   view = GTK_TEXT_VIEW (widget);
   if (!gtk_text_view_get_editable (view))
     return;
-  buffer = view->buffer;
+  buffer = gtk_text_view_get_buffer (view);
 
   gtk_text_buffer_get_iter_at_offset (buffer, &start, start_pos);
   gtk_text_buffer_get_iter_at_offset (buffer, &end, end_pos);
@@ -1295,7 +1285,7 @@ gail_text_view_delete_text (AtkEditableText *text,
   GtkTextIter start_itr;
   GtkTextIter end_itr;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return;
@@ -1303,7 +1293,7 @@ gail_text_view_delete_text (AtkEditableText *text,
   view = GTK_TEXT_VIEW (widget);
   if (!gtk_text_view_get_editable (view))
     return;
-  buffer = view->buffer;
+  buffer = gtk_text_view_get_buffer (view);
 
   gtk_text_buffer_get_iter_at_offset (buffer, &start_itr, start_pos);
   gtk_text_buffer_get_iter_at_offset (buffer, &end_itr, end_pos);
@@ -1320,7 +1310,7 @@ gail_text_view_paste_text (AtkEditableText *text,
   GailTextViewPaste paste_struct;
   GtkClipboard *clipboard;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return;
@@ -1328,7 +1318,7 @@ gail_text_view_paste_text (AtkEditableText *text,
   view = GTK_TEXT_VIEW (widget);
   if (!gtk_text_view_get_editable (view))
     return;
-  buffer = view->buffer;
+  buffer = gtk_text_view_get_buffer (view);
 
   paste_struct.buffer = buffer;
   paste_struct.position = position;
@@ -1551,7 +1541,7 @@ get_text_near_offset (AtkText          *text,
   GtkTextView *view;
   gpointer layout = NULL;
 
-  view = GTK_TEXT_VIEW (GTK_ACCESSIBLE (text)->widget);
+  view = GTK_TEXT_VIEW (gtk_accessible_get_widget (GTK_ACCESSIBLE (text)));
 
   /*
    * Pass the GtkTextView to the function gail_text_util_get_text() 
